unit tecnicas;

{$mode objfpc}{$H+}

interface

uses
  dibujos, VB,
  DynLibs,
  Dialogs,
  Forms, Classes, SysUtils, strutils, Process;

type
  // aspectados = array[0..29,0..29] of integer;

  {$IFDEF Windows}
  swe_calc   =  function(tjd : double;    {Julian day number}
                 ipl         : Integer;   {planet number}
                 iflag       : Longint;   {flag bits}
                 var xx      : double;    {first of 6 doubles}
                 sErr        : PChar      {Error-String}
  ): Longint; stdcall;
  swe_houses = function
                   (tjdut      : double;
                    geolat     : double;
                    geolon     : double;
                    hsys       : char;
                    var hcusp0 : double;
                    var ascmc0 : double
) : integer; stdcall; // far; external 'swedll32.dll' name '_swe_houses@36';
  {$ELSE}
  swe_calc   =  function(tjd : double;    {Julian day number}
                 ipl         : Integer;   {planet number}
                 iflag       : Longint;   {flag bits}
                 var xx      : double;    {first of 6 doubles}
                 sErr        : PChar      {Error-String}
  ): Longint; cdecl;
  swe_houses = function
                   (tjdut      : double;
                    geolat     : double;
                    geolon     : double;
                    hsys       : char;
                    var hcusp0 : double;
                    var ascmc0 : double
  ) : integer; cdecl;

  {$ENDIF}



procedure ListaTransitos(TrMenores, SoloLentos: boolean; diasTrans, fechaFuturo: string; longi: LongPl; declp: DeclinPl; interpreta: boolean);
procedure ListaPrimarias(diasTrans, fechaFuturo: string; longi: LongPl; lap, longeo, latgeo:double);
function swCalcula(cadena: string): string;
function Obliq(djd: Double): Double;
function Lapso(aa, mm, dd, hh, mn, ss : double): double;
function Juliano_Actual(var jd: double): string;
function interpreta_transitos(aspe: longint; t, n: integer; longi: LongPl): string;

const
  Pruebas = False; // True;  // ********* PRUEBAS *********
  SEFLG_JPLEPH : Longint =    	1;       { use JPL ephemeris }
  SEFLG_SWIEPH : Longint =    	2;       { use SWISSEPH ephemeris }
  SEFLG_MOSEPH : Longint =    	4;       { use Moshier ephemeris }
  SEFLG_SPEED  : Longint =      256;
  SEFLG_EQUATORIAL : Longint = 2048;
implementation

function Lapso(aa, mm, dd, hh, mn, ss : double): double;
var
  djd : double;
  ti, tr, tj, n1, d4 : Double;
  L, Imx, aa2, DayW : Double;
begin
  If aa < 0 Then
  begin
    aa2:= aa + 1;
  end
  Else
  begin
    aa2 := aa;
  end;
  ti := 365 * aa2 + dd + 31 * (mm - 1);
  If mm < 3 Then begin
    ti := ti + Int((aa2 - 1) / 4 - Int(Int((aa2 - 1) / 100 + 1) * 3 / 4));
  end;
  tr := ti - Int(0.4 * mm + 2.3) + Int(aa2 / 4) - Int((Int(aa2 / 100) + 1) * 3 / 4);
  If mm >= 3 Then begin
    ti := tr;
  end;
  tj := ti - 693960;
  n1 := tj - 365;
  d4 := n1 + 1;
  DayW := Int(d4 - 7 * Int(d4 / 7));
  If DayW = 0 Then begin
    DayW := 7;
  end;
  // PRINT " "; MID$("lunmarmiejueviesabdom", 3 * dayw - 2, 3)
  L := n1 + (hh + (mn + ss / 60) / 60) / 24;
  // Lapso := L;
  djd := L + 364.5;
  If djd < -115848 Then begin
    Imx := 12 * (aa2 + 4800) + mm - 3;
    djd := djd - (trunc(Imx / 4800) - trunc(Imx / 1200) + 38);
  End;
  Lapso := djd + 2415020;
  // t = djd / 36525
End;

function Juliano_Actual(var jd: double): string;
var
  L, n , IT : single;
  Kdia, Kmes, Kano: string;
  Jmes, Jdia, Jano: single;
begin
  L := trunc(jd + 0.5) + 68569;
  n := trunc(4 * L / 146097);
  L := L - trunc((146097 * n + 3) / 4);
  IT := trunc(4000 * (L + 1) / 1461001);
  L := L - trunc(1461 * IT / 4) + 31;
  Jmes := trunc(80 * L / 2447);
  Jdia := L - Int(2447 * Jmes / 80);
  L := trunc(Jmes / 11);
  Jmes := Jmes + 2 - (12 * L);
  Jano := 100 * (n - 49) + IT + L;
  Str(trunc(Jdia), Kdia);
  If Jdia < 10 Then Kdia := '0' + RightStr(Kdia, 1);
  Str(trunc(Jmes), Kmes);
  If Jmes < 10 Then Kmes := '0' + RightStr(Kmes, 1);
  Str(trunc(Jano),Kano);
  //Kano := RightStr(Kano, 4);
  Juliano_Actual:= Trim(Kdia) + '/' + Trim(Kmes) + '/' + Trim(Kano);
End;

function Obliq(djd: Double): Double;
 var
   t, c, eps: Double;
 begin
   t := (djd - 2415020) / 36525;
   c := (((-0.00181 * t) + 0.0059) * t + 46.845) * t;
   eps := 23.45229444 - (c / 3600);
   Obliq := eps;
End;

function swCalcula(cadena: string): string;
var
  s, st: string;
  AProcess: TProcess;
  AStringList: TStringList;
  td: array of string;
  i: integer;
begin
   AProcess := TProcess.Create(nil);
   AStringList := TStringList.Create;
   AProcess.CommandLine := cadena;
   AProcess.Options := AProcess.Options + [poWaitOnExit, poUsePipes, poNoConsole];
   AProcess.Execute;

   if AProcess.Output.NumBytesAvailable>0 then begin
     SetLength(st, AProcess.Output.NumBytesAvailable);
     AProcess.Output.ReadBuffer(st[1], AProcess.Output.NumBytesAvailable);
   end;
   td := vbSplit(st,chr(10));
   s:='';
   for i:= 0 to (length(td))-1 do begin
     s:= s + ',' + td[i];
   end;
   FreeAndNil(AStringList);
   FreeAndNil(AProcess);
   swCalcula:= s;
end;

function aspecto_entero(dist : string) : integer;
  var
    d, n : integer;
  begin
  d:= vbVal(dist);
  Case d of
  0 : n:= 1;
  180: n:= 2;
  120,240: n:= 3;
  90,270: n:= 4;
  72,144: n:= 5;
  60, 300: n:= 6;
  51,102,154: n:=7; //Parte entera
  45,135: n:= 8;
  40,80,160: n:= 9;
  36,108: n:= 10;
  32,65,98,130,163: n:= 11;
  30,330,150,210: n:= 12;
  End;
  aspecto_entero := n;
End; // Function

function aspecto_transit_declin(orbe, transitante, transitado: single; var distancia: single): integer;
  var
  Aspecto: integer;
  begin
  distancia := abs(abs(transitante) - abs(transitado));
  if (distancia < (orbe)) then begin
    if ((transitante > 0) and (transitado > 0)) or
       ((transitante < 0) and (transitado < 0)) then begin
         Aspecto := 0; // Paralelo
       end
       else begin
         Aspecto := 1; // Antiparalelo
       end;
  end
  else begin
    Aspecto := -1;
  end;
  aspecto_transit_declin:= Aspecto;
end;

function aspecto_transit(TrMenores: boolean; orbe, transitante, transitado: single; var distancia: single): integer;
  var
  Aspecto: integer;
  begin
    Aspecto := -1;
    distancia := transitante - transitado;
    if (distancia < 0) then distancia := distancia + 360;
    if (distancia > 180) then distancia := 360 - distancia;

    if (distancia < (orbe)) then begin
        Aspecto := 0;
    end
    else if ((180 - (orbe)) <= distancia) and (distancia <= 180) then begin
        Aspecto := 180;
    end
    else if ((120 - orbe) <= distancia) and (distancia <= (120 + orbe)) then begin
        Aspecto := 120;
    end
    else if ((90 - orbe) <= distancia) and (distancia <= (90 + orbe)) then begin
        Aspecto := 90;
    end
    else if ((60 - orbe) <= distancia) and (distancia <= (60 + orbe)) then begin
        Aspecto := 60;
    end
    else if ((30 - orbe) <= distancia) and (distancia <= (30 + orbe)) then begin
        Aspecto := 30;
    end
    else if ((150 - orbe) <= distancia) and (distancia <= (150 + orbe)) then begin
        Aspecto := 150;
    end
    else begin
        Aspecto := - 1;
    end;
    // *** Menores ***
    if (aspecto = -1) and (TrMenores) then begin // and (Pruebas = True) then begin
      if ((45 - orbe) <= distancia) and (distancia <= (45 + orbe)) then begin
        Aspecto := 45;
      end
      else if ((135 - orbe) <= distancia) and (distancia <= (135 + orbe)) then begin
        Aspecto := 135;
      end
      else if ((72 - orbe) <= distancia) and (distancia <= (72 + orbe)) then begin
        Aspecto := 72;
      end
      else if ((144 - orbe) <= distancia) and (distancia <= (144 + orbe)) then begin
        Aspecto := 144;
      end;
    end;
    aspecto_transit:= Aspecto;
end;

function swPlaneta(trn: integer; dia, mes, anio: string): single;
var
  s, npl, txt : string;
  lonstr: array of string;
  // salida: ansistring;
begin
    Str(trn, npl);
    txt:= Concat('swetest -p', npl, ' -fl -b', dia,'.', mes,'.', anio);
    txt:= Concat(txt, ' -t', '00','.', '00', '00',' -head '); // hh,mm,ss
    txt:= Concat(txt , '-emos '); // Moshier
    s:= swCalcula(txt);
    lonstr:= vbSplit(s,',');
    swPlaneta:= vbVal(Trim(lonstr[1]));
end;

function swAvance(trn: integer; dia, mes, anio: string): single;
var
  s, npl, txt : string;
  lonstr: array of string;
begin
    Str(trn, npl);
    txt:= Concat('swetest -p', npl, ' -fs -b', dia,'.', mes,'.', anio);
    txt:= Concat(txt, ' -t', '00','.', '00', '00',' -head '); // hh,mm,ss
    txt:= Concat(txt , '-emos '); // Moshier
    s:= swCalcula(txt);
    lonstr:= vbSplit(s,',');
    swAvance:= vbVal(Trim(lonstr[1]));
end;

function hayAspectoPeriodo(rad, tra, avance: single; aspecto: integer): boolean;
var
  si: boolean;
  dis: single;
begin
    si := False;
    if(rad > tra) then begin
        dis := rad - tra;
        if (dis > 190) then tra := tra + 360;
    end
    else begin
        dis := tra - rad;
        if (dis > 190) then rad := rad + 360;
    end;
    if (rad > tra) then
        tra := tra + aspecto
    else rad := rad + aspecto;

    if (tra >= rad) and ((tra+avance) <= rad) then
        si := True
    else if (tra <= rad) and ((tra + avance) >= rad) then si := True;
    hayAspectoPeriodo:= si;
end;

procedure ListaPrimarias(diasTrans, fechaFuturo: string; longi: LongPl; lap, longeo, latgeo:double);
var
  f : TextFile;
  cadf : array of string;
  nombrepl, dt: array of string;
  s, txt, Fecha, dia, mes, anio, cadena, cadant, saspecto: string;
  lp, nat, an, me, di, distancia, distanciab, falta, avance: single;
  falta1, falta2 : single;
  jdb, julday, jd: double;
  orbe, dia1, diad, hasta, lb, la: single;
  dias, dia2, apt, fTope, tr, nt, ntl, trn, aspecto, aspectob : integer;
  aprox, i, j, k, n : integer;
  masexacto, archiva : boolean;
  res: longint;
  astro_Handle:TLibHandle=dynlibs.NilHandle;
  swHous: swe_houses;
  hcusps: array[0..12] of double;
  ascmc: array[0..9] of double;
begin
  s:= Application.Location;
  {$IFDEF Windows}
    astro_Handle := LoadLibrary(s + 'swedll32.dll');
    swHous := swe_houses(GetProcedureAddress(astro_Handle, '_swe_houses@36'));
  {$ELSE}
    astro_Handle := LoadLibrary(s + 'libswe.so');
    swHous := swe_houses(GetProcedureAddress(astro_Handle, 'swe_houses'));
  {$ENDIF}
  n:= 0;
  cadant := ' ';
  s:= 'Sol,Lun,Mer,Ven,Mar,Jup,Sat,Ura,Nep,Plu,Nod,Asc,M.C';
  nombrepl:= vbSplit(s, ',');
  dt:= vbSplit(fechaFuturo,'/');
  Val(dt[2],di);
  Val(dt[1],me);
  Val(dt[0],an);
  Val(diasTrans, fTope);
  julday:=lapso(an, me, di, 0.0, 0.0, 0.0);
  orbe:= 0.025;
  i:=0;
  for dias:= 0 to fTope do begin
    //jd := (dias * 365.24219907) + julday;
    jd := dias + julday;
    Fecha:= Juliano_Actual(jd);
    // Primarias de As y Mc
    // jd := Trunc(dias * 0.958) + Trunc(julday * 0.000007496);
    // jd := Trunc(dias * 0.958) + Trunc((julday - lap) * 0.000007496);
    // jd := (dias * 0.958) + ((julday - lap) * 0.000007181);
    jd := ((dias + (julday-lap)) * 0.000007496);
    jd := jd + lap;
    dt:= vbSplit(Fecha, '/');
    //dia:= dt[0];
    //mes:= dt[1];
    //anio:= dt[2];
    for tr:= 11 to 12 do begin // As a Mc
      trn:= tr - 2; // *** Adaptandonos a Swiss Ephemeris
      //res:= swHous(lap + jd, latgeo, longeo, 'T', hcusps[0], ascmc[0]);
      res:= swHous(jd, latgeo, longeo, 'T', hcusps[0], ascmc[0]);
      if (tr = 11) then lp:= hcusps[1] else lp:= hcusps[10];
      avance :=1; // ???
      for nt:= 0 to 10 do begin // Planetas natales
        ntl:=nt;
        nat:=longi[0,ntl];
        aspecto:= aspecto_transit(false, orbe, lp, nat,  distancia);
        falta := abs(aspecto - distancia);
        if (aspecto > -1) then begin
          aprox :=0;
          aspectob:= aspecto_transit(false, orbe, la, nat,  distancia);
          if aspectob > -1 then begin
            falta1 := abs(aspectob - distancia);
          end
          else begin
            falta1 :=-1;
          end;
          if (hayAspectoPeriodo(nat, lp, avance, aspecto)) then begin
            masexacto:= true;
            str(aspecto, saspecto);
            jdb:= julday + dias - aprox; // Exacto
            cadena:= Fecha;
            txt:= nombrepl[tr] + ' ' + saspecto + ' ' + nombrepl[nt];
            if (cadant = txt) then archiva:= False // Evita repeticiones
              else archiva := True;
            cadant:= txt;
            cadena:= Concat(cadena,' ', txt);
            SetLength(cadf, i + 1);
            cadf[i]:= cadena;
            inc (i);
          end;
        end;
      end;
    end;
  end;// dias

  k := length(cadf); // Eliminar repetidas
  for i:= 0 to k-2 do begin
   s := cadf[i];
   if s <> '' then begin
     for j:= (i + 1) to k-1 do begin
       txt:= cadf[j];
       if txt <> '' then begin
         if (txt = s) then cadf[j]:='';
       end;
     end;
   end;
  end;

  AssignFile( f, ExtractFilePath( Application.ExeName ) + 'transitos.txt' );
  Rewrite(f); // escritura
  for i:= 0 to k-1 do begin
    if cadf[i] <> '' then writeln(f, cadf[i]);
  end;
  CloseFile(f);

  UnloadLibrary(astro_Handle);
end;

Function horaAspecto(falta: single; avance : double; var jd: double): String;
  var
  x: single;
  Hora : String;
  begin
  x := (24 * falta) / Abs(avance);
  While (x < 0) do begin
      jd := jd - 1;
      x := x + 24;
  end;
  While (x > 24) And (x < 48) do begin
    jd := jd + 1;
    x := x - 24;
  end;
  If (Abs(x) > 48) Then begin
    horaAspecto := 'aprox';
  end
  Else begin
    Hora:= TimeToStr(x/24);
    {$IFDEF Windows}
    if (x < 10) then begin
      Hora:= '0' + Hora;
    end;
    {$ENDIF}
    Hora := VbLeft(Hora, 5);
    horaAspecto := Hora;
  end;
End;

procedure ListaTransitos(TrMenores, SoloLentos: boolean; diasTrans, fechaFuturo: string; longi: LongPl; declp: DeclinPl; interpreta: boolean);
var
  f : TextFile;
  cadf, tiempo : array of string;
  cdInterp: array of string;
  nombrepl, dt: array of string;
  s, txt, Fecha, dia, mes, anio, cadena, cadant, saspecto, txItp: string;
  horasp1, horasp2, horasp: string;
  lp, nat, an, me, di, distancia, distanciab, falta, avance: single;
  falta1, falta2 : single;
  jdbak, jdb, julday, jdbLuna, jd: double;
  orbe, orbde, dia1, diad, hasta, lb, la: single;
  dias, dia2, apt, fTope, tr, nt, ntl, trn, aspecto, aspectob, tr1, tr2, na1: integer;
  aprox, i, j, k, n, tecn, tec2 : integer;
  masexacto, archiva, usaLuna : boolean;

  astro_Handle:TLibHandle=dynlibs.NilHandle;
  swCalc: swe_calc;

  res: longint;
  xx : array [0..5] of double;
  sErr: array[0..255] of char;

  begin
  s:= Application.Location;
  {$IFDEF Windows}
    astro_Handle := LoadLibrary(s + 'swedll32.dll');
    swCalc := swe_calc(GetProcedureAddress(astro_Handle, '_swe_calc@24'));
  {$ELSE}
    astro_Handle := LoadLibrary(s + 'libswe.so');
    swCalc := swe_calc(GetProcedureAddress(astro_Handle, 'swe_calc'));
  {$ENDIF}
  n:= 0;
  cadant := ' ';
  s:= 'Sol,Lun,Mer,Ven,Mar,Jup,Sat,Ura,Nep,Plu,Nod,Asc,M.C';
  nombrepl:= vbSplit(s, ',');
  fechaFuturo:= StringReplace(fechaFuturo, '-', '/', [rfReplaceAll, rfIgnoreCase]);
  dt:= vbSplit(fechaFuturo,'/');
  Val(dt[2],di);
  Val(dt[1],me);
  Val(dt[0],an);
  Val(diasTrans, fTope);
  julday:=lapso(an, me, di, 0.0, 0.0, 0.0);
  if (fTope < 8) then orbe:= 2.0 else orbe:= 1.0;
  tr2:=12;
  if (fTope >= 999) then begin // Para rectificacion
    tr1:= 7; // Jup a Plu
    na1:= 11;
  end
  else if (SoloLentos = True) then begin // Lo importante del año
    tr1:=9;
    tr2:=11;
    na1:=0;
  end
  else begin
    tr1:= 2;
    na1:=0;
  end;
  if (fTope < 4) then begin
    usaLuna := true;
  end
  else begin
    usaLuna := false;
  end;
  if TrMenores then tec2:= 2 else tec2:=1;
  for dias:= 0 to fTope do begin
    jd := dias + julday;
    Fecha:= Juliano_Actual(jd);
    dt:= vbSplit(Fecha, '/');
    dia:= dt[0];
    mes:= dt[1];
    anio:= dt[2];
    for tecn:= 1 to tec2 do begin // 1 Ecliptica 2 Declinacion
      for tr:= tr1 to tr2 do begin // Mer=4 a Plu=11 (Nodo 12)
        trn:= tr - 2; // *** Adaptandonos a Swiss Ephemeris
        // Lo siguiente da fallo en Linux si las swe no estan instaladas:
        if (tecn = 1) then begin
          res:= swCalc(jd,trn,SEFLG_MOSEPH + SEFLG_SPEED,xx[0],sErr);
          lp:= xx[0];
          avance:= xx[3];
        end
        else begin
          res:= swCalc(jd,trn,SEFLG_MOSEPH + SEFLG_SPEED + SEFLG_EQUATORIAL,xx[0],sErr);
          lp:= xx[1];
          avance:= xx[4];
        end;
        longi[1,trn]:= lp; // !!! Para interpretacion de futuro
        // 0.242 Jup (0.75) (0.5)
        // Solo av. max. de Jup. (salvo para menos de 8 dias)
        If (Abs(avance) < 0.5) Or (ftope < 8) Then begin
          if (tecn = 1) then begin
            If (avance > 8) Then begin // Luna
              orbe := 14.9;
            end
            Else If (avance < 8) And (ftope < 8) Then begin
              orbe := 2;
            end
            Else If (avance > 8) And (ftope >= 8) Then begin
              orbe := 1;
            End;
          end
          else begin // !!! Orbe declinacion
            if (fTope < 8) then orbde:= 0.17 else orbde:= 0.25; // 0.17 y 0.083
          end;
          for nt:= na1 to 12 do begin
            ntl:=nt;
            if (nt = 12) then ntl:=20; // M.C.
            if (tecn = 1) then begin
              nat:=longi[0,ntl];
              aspecto:= aspecto_transit(TrMenores, orbe, lp, nat,  distancia);
              falta := abs(aspecto - distancia);
            end
            else begin
              nat:=declp[ntl];
              aspecto:= aspecto_transit_declin(orbde, lp, nat,  distancia);
              falta := distancia;
            end;
            if (aspecto > -1) then begin
              aprox :=0;
              if (tecn = 1) then begin
                res:= swCalc(jd-1.0,trn,SEFLG_MOSEPH,xx[0],sErr);
                la:= xx[0];
                aspectob:= aspecto_transit(TrMenores, orbe, la, nat,  distancia);
                if aspectob > -1 then begin
                  falta1 := abs(aspectob - distancia);
                end
                else begin
                  falta1 :=-1;
                end;
                res:= swCalc(jd+1.0,trn,SEFLG_MOSEPH,xx[0],sErr);
                lb:= xx[0];
                aspectob:= aspecto_transit(TrMenores, orbe, lb, nat,  distancia);
                if aspectob > -1 then begin
                  falta2 := abs(aspectob - distancia);
                end
                else begin
                  falta2 :=-1;
                end;
              end
              else begin
                res:= swCalc(jd-1.0,trn,SEFLG_MOSEPH + SEFLG_EQUATORIAL,xx[0],sErr);
                la:= xx[1];
                aspectob:= aspecto_transit_declin(orbde, la, nat,  distancia);
                if aspectob > -1 then begin
                  falta1 := distancia;
                end
                else begin
                  falta1 :=-1;
                end;
                res:= swCalc(jd+1.0,trn,SEFLG_MOSEPH + SEFLG_EQUATORIAL,xx[0],sErr);
                lb:= xx[1];
                aspectob:= aspecto_transit_declin(orbde, lb, nat,  distancia);
                if aspectob > -1 then begin
                  falta2 := distancia;
                end
                else begin
                  falta2 :=-1;
                end;
              end;
              masexacto := false;
              if (tecn = 1) then begin
                if (hayAspectoPeriodo(nat, lp, avance, aspecto)) then
                  masexacto:= true;
              end
              else begin
                if (hayAspectoPeriodo(nat, lp, avance, 0)) then
                  masexacto:= true;
              end;

              if (masexacto = false) then begin
                if (falta1 > -1) and (falta2 > -1) then begin
                  if (falta <= falta1) and (falta <= falta2) then begin
                    aprox:= 1;
                    masexacto:=true;
                  end;
                end;
              end;

              if (masexacto = true) then begin
                dia2:= 0; // *** Final ***
                aspectob:= aspecto;
                while (aspectob >-1) do begin
                  dia2:= dia2+1;
                  diad:=dia2 + dias;
                  trn:= tr -2; // Para SwEph
                  jdb:=julday + diad;
                  if (tecn = 1) then begin
                    res:= swCalc(jdb,trn,SEFLG_MOSEPH,xx[0],sErr);
                    lb:= xx[0];
                    aspectob:= aspecto_transit(TrMenores, orbe, lb, nat, distanciab);
                  end
                  else begin
                    res:= swCalc(jdb,trn,SEFLG_MOSEPH + SEFLG_EQUATORIAL,xx[0],sErr);
                    lb:= xx[1];
                    aspectob:= aspecto_transit_declin(orbde, lb, nat, distanciab);
                  end;
                end; // while
                // dia1:= dias - dia2;
                hasta:= diad;
                diad:= dias;
                if (tecn = 1) then begin
                  str(aspecto, saspecto);
                end
                else begin
                  if (aspecto = 0) then saspecto:= '999' else saspecto:='909';
                end ;
                jdb:=julday + diad - aprox; // *** Exacto ***
                horasp := horaAspecto(falta, avance, jdb);

                // *** Hora de la Luna ***
                {
                jdbak := jdb;
                if (avance > 8) then begin
                  horasp1 := horaAspecto(falta-1, avance, jdb);
                  horasp2 := horaAspecto(falta+1, avance, jdb);
                end;
                jdb := jdbak;
                }
                cadena:= Juliano_Actual(jdb);
                jdbLuna := jdb;
                txt:= nombrepl[trn] + ' ' + saspecto + ' ' + nombrepl[nt];
                if (cadant = txt) then archiva:= False // Evita repeticiones
                  else archiva := True;
                cadant:= txt;
                cadena:= Concat(cadena,' ', txt);

                dia2:= 0; // *** Comienzo ***
                aspectob:= aspecto;
                while (aspectob >-1) do begin
                  dia2:= dia2-1;
                  diad:=dia2 + dias;
                  trn:= tr -2; // Para SwEph
                  jdb:=julday + diad;
                  if (tecn = 1 ) then begin
                    res:= swCalc(jdb,trn,SEFLG_MOSEPH,xx[0],sErr);
                    lb:= xx[0];
                    aspectob:= aspecto_transit(TrMenores, orbe, lb, nat, distanciab);
                  end
                  else begin
                    res:= swCalc(jdb,trn,SEFLG_MOSEPH + SEFLG_EQUATORIAL,xx[0],sErr);
                    lb:= xx[1];
                    aspectob:= aspecto_transit_declin(orbde, lb, nat, distanciab);
                  end;
                end; // while
                dia1 := diad;
                jdb:=julday + dia1;
                if (avance > 8) then jdb := jdbLuna;
                {
                if (avance > 8) then begin // Luna
                  cadena:= Concat(cadena, ' (', horasp1);
                end
                else begin
                }
                  cadena:= Concat(cadena, ' (', vbLeft(Juliano_Actual(jdb),5));
                //end;

                jdb:=julday + hasta;
                if (avance > 8) then jdb := jdbLuna;
                {
                if (avance > 8) then begin // Luna
                  cadena:= Concat(cadena, '>', horasp2,')');
                end
                else begin}
                  cadena:= Concat(cadena, '>', vbLeft(Juliano_Actual(jdb),5), ')');
                //end;
                // Ver si repite natal:
                if (tecn = 1) then begin
                  apt:= calcula_aspecto(longi, trn, ntl, False,0);
                end
                else begin
                  apt:= calcula_asparal(declp, trn, ntl, False, orbde);
                end;
                if apt = -1 then begin
                  txt :='   (.) ';
                end
                else begin
                  Str(apt,txt);
                  txt := '   (' + txt +') ';
                end;
                cadena := cadena + txt + ' ' + horasp;
                if (archiva) then begin
                    Setlength(cadf, n+1);
                    Setlength(tiempo, n+1);
                    cadf[n]:= cadena;
                    s:=horasp;
                    if (horasp = 'aprox') then horasp := '00:00';
                    tiempo[n]:= vbVal(vbMid(cadena, 7,4) + vbMid(cadena,4,2)+vbleft(cadena,2)+vbLeft(horasp,2)+vbRight(horasp,2));
                    horasp :=s;
                    if (interpreta) then begin
                      txt:= interpreta_transitos(aspecto, trn, ntl, longi);
                      if (txt <>'') then
                        txItp := txItp + cadena + txt; //  <<<
                    end;
                    inc(n);
                end;
              end;
            end;
          end;
        end;
      end;
    end;// tecn
  end;// dias

  k := length(cadf); // Eliminar repetidas
  for i:= 0 to k-2 do begin
   s := cadf[i];
   if s <> '' then begin
     for j:= (i + 1) to k-1 do begin
       txt:= cadf[j];
       if txt <> '' then begin
         n := Length(txt) - 5; // -26 para las horas de la Luna
         if (VbLeft(txt,n) = VbLeft(s,n)) then cadf[j]:=''; // Comparar sin hora
       end;
     end;
   end;
  end;

// Ordenar por tiempo
For i := high(cadf) downto low(cadf) do begin
  For j := low(cadf) + 1 To i do begin
    If tiempo[j - 1] > tiempo[j] Then begin
      txt := cadf[j - 1];
      s := tiempo[j - 1];
      cadf[j - 1] := cadf[j];
      tiempo[j - 1] := tiempo[j];
      cadf[j] := txt;
      tiempo[j] := s;
    end;
  end;
end;

  AssignFile( f, ExtractFilePath( Application.ExeName ) + 'transitos.txt' );
  Rewrite(f); // escritura
  for i:= 0 to k-1 do begin
    if cadf[i] <> '' then begin
      writeln(f, cadf[i]);
    end;
  end;
  CloseFile(f);

  UnloadLibrary(astro_Handle);
end;

function interpreta_transitos(aspe: longint; t, n: integer; longi: LongPl): string;
  var
    s: string;
  begin
  // Mejor un bucle que lea distintas frases aleatorias con significado
  // equivalente para cada posibilidad. Piensa en Venus en amor, economia, etc...
    s:= '';
    if (t = 7) and (n < 5) then begin // Urano transitando
      s:= s + 'Estás viviendo una época ideal para liberar e innovar ';
    end
    else if (t = 8) and (n < 5) then begin // Neptuno transitando
      s:= s +  'Te encuentras experimentando un tiempo favorable para sintonizar, espiritualizar y empatizar con ';
    end
    else if (t = 9) and (n < 5) then begin // Pluton transitando
      s:=  s + 'Atraviesas un periodo idóneo para desapegarte y quedarte con lo esencial de ';
    end;
    if ( t > 6) and (t < 10) then begin
      if (n = 0) then begin // Sol
        s:=  s + 'tus verdaderos objetivos, tu poder y autoridad.';
      end
      else if (n = 1) then begin // Luna
        s:=  s + ' tus sentimientos, tus recuerdos dependencias infantiles de ti hacia los demás y de los demás hacia ti.';
      end
      else if (n = 2) then begin // Mercurio
        s:=  s + ' tu lógica, tu pensamiento y manera de comunicarte.';
      end
      else if (n = 3) then begin // Venus
        s:=  s + ' tu vida afectiva, tu sentido estético, tus gustos y placeres.';
      end
      else if (n = 4) then begin // Marte
        s:=  s + ' tu capacidad de actuar, defenderte, luchar y conquistar.';
      end
      else if (n = 12) then begin // Ascendente
        s:= s + 'tu personalidad, tu vida.';
      end
      else if (n = 22) then begin // Medio Cielo
        s:= s + 'tu profesión, vocación y responsabilidades';
      end;
    end;
    // Urano /alt-1: Estás liberándote de alt-2 Estás volviéndote muy original y libre
    // Casas: 'Especialmente con tus amigos/profesión/pareja/economía/hijos...
    interpreta_transitos:=s;
end;

end.

